%%%-------------------------------------------------------------------
%%% File    : dialyzer_codeserver.erl
%%% Author  : Tobias Lindahl <tobiasl@it.uu.se>
%%% Description : 
%%%
%%% Created :  4 Apr 2005 by Tobias Lindahl <tobiasl@it.uu.se>
%%%-------------------------------------------------------------------
-module(dialyzer_codeserver).

-export([all_exports/1, 
	 insert/3, 
	 insert_exports/2,	 
	 lookup/3, 
	 new/0,
	 next_core_label/1,
	 update_next_core_label/2]).


-record(dialyzer_codeserver, {table, exports, next_core_label}).

new() ->
  Table = table__new(),
  Exports = ets:new(dialyzer_codeserver_exports, []),
  #dialyzer_codeserver{table=Table, exports=Exports, next_core_label=0}.

insert(List, Tag, CS) ->
  List1 = [{{MFA, Tag}, Code}||{MFA, Code} <- List],
  table__insert(CS#dialyzer_codeserver.table, List1),
  CS.

insert_exports(List, CS = #dialyzer_codeserver{exports=Ets}) ->
  true = ets:insert(Ets, [{MFA, export} || MFA <- List]),
  CS.

all_exports(#dialyzer_codeserver{exports=Ets}) ->  
  List = [MFA || {MFA, export} <- ets:match_object(Ets, {'_',export})],
  sets:from_list(List).

lookup(MFA, Tag, CS) ->
  case table__lookup(CS#dialyzer_codeserver.table, {MFA, Tag}) of
    [{{MFA, Tag}, Val}] -> {ok, Val};
    _ -> error
  end.
      

next_core_label(#dialyzer_codeserver{next_core_label=NCL}) ->
  NCL.

update_next_core_label(CS = #dialyzer_codeserver{}, NCL) ->
  CS#dialyzer_codeserver{next_core_label=NCL}.


table__new() ->
  ets:new(dialyzer_codeserver, []).

table__lookup(Table, Key) ->
  ets:lookup(Table, Key).

table__insert(Table, List) ->
  true = ets:insert(Table, List).
