%%%-------------------------------------------------------------------
%%% File    : dialyzer_test_plt.erl
%%% Author  : Tobias Lindahl <tobiasl@csd.uu.se>
%%% Description : Utility to do different checks on the persistent lookup table.
%%%
%%% Created : 23 Nov 2005 by Tobias Lindahl <tobiasl@csd.uu.se>
%%%-------------------------------------------------------------------
-module(dialyzer_test_plt).

-export([compare/2, dump_types/2]).

compare(Plt1, Plt2) ->
  Ets1 = dialyzer_plt:from_dets(ets1, Plt1),
  Keys1 = lists:sort(ets:match_object(Ets1, '_')),
  Ets2 = dialyzer_plt:from_dets(ets2, Plt2),
  Keys2 = lists:sort(ets:match_object(Ets2, '_')),
  compare_1(Keys1, Keys2, Plt1, Plt2).
  

compare_1([H|T1], [H|T2], Plt1, Plt2) ->
  compare_1(T1, T2, Plt1, Plt2);
compare_1([H1|T1], L2 = [H2|_], Plt1, Plt2) when H1 < H2 ->
  io:format("In ~p but not in ~p:\n", [Plt1, Plt2]),
  print_formatted_diff(H1),
  compare_1(T1, L2, Plt1, Plt2);
compare_1(L1, [H2|T2], Plt1, Plt2)  ->
  io:format("In ~p but not in ~p:\n", [Plt2, Plt1]),
  print_formatted_diff(H2),
  compare_1(L1, T2, Plt1, Plt2);
compare_1([], [], _Plt1, _Plt2) ->
  ok;
compare_1(L1, [], Plt1, Plt2) ->
  io:format("In ~p but not in ~p:\n", [Plt1, Plt2]),
  [print_formatted_diff(X) || X <- lists:reverse(L1)],
  ok;
compare_1([], L2, Plt1, Plt2) ->
  io:format("In ~p but not in ~p:\n", [Plt2, Plt1]),
  [print_formatted_diff(X) || X <- lists:reverse(L2)],
  ok.

print_formatted_diff({MFA, Return, Arg}) ->
  Fun = erl_types:t_fun(Arg, Return),
  io:format("\t~p :: ~s\n", [MFA, erl_types:t_to_string(Fun)]);
print_formatted_diff(Other) ->
  io:format("\t~p\n", [Other]).
  
dump_types(PltFile, OutputFile) ->
  Plt = dialyzer_plt:from_dets(dialyzer_test_plt, PltFile),
  List = ets:tab2list(Plt),
  AllTypes = [erl_types:t_fun(Arg, Ret) || {_, Ret, Arg} <- List],
  file:write_file(OutputFile, term_to_binary(AllTypes)).
